@@ -22,15 +22,19 @@ public:
   RooGamma() {} ;
   RooGamma(const char *name, const char *title,
          RooAbsReal& _x, RooAbsReal& _gamma, RooAbsReal& _beta, RooAbsReal& _mu);
-  RooGamma(const RooGamma& other, const char* name=0) ;
-  virtual TObject* clone(const char* newname) const { return new RooGamma(*this,newname); }
-  inline virtual ~RooGamma() { }
+  RooGamma(const RooGamma& other, const char* name=nullptr) ;
+  TObject* clone(const char* newname=nullptr) const override { return new RooGamma(*this,newname); }
 
-  Int_t getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVars, const char* rangeName=0) const ;
-  Double_t analyticalIntegral(Int_t code, const char* rangeName=0) const ;
+  Int_t getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVars, const char* rangeName=nullptr) const override ;
+  double analyticalIntegral(Int_t code, const char* rangeName=nullptr) const override ;
 
-  Int_t getGenerator(const RooArgSet& directVars, RooArgSet &generateVars, Bool_t staticInitOK=kTRUE) const;
-  void generateEvent(Int_t code);
+  Int_t getGenerator(const RooArgSet& directVars, RooArgSet &generateVars, bool staticInitOK=true) const override;
+  void generateEvent(Int_t code) override;
+
+  RooAbsReal const &getX() const { return *x; }
+  RooAbsReal const &getGamma() const { return *gamma; }
+  RooAbsReal const &getBeta() const { return *beta; }
+  RooAbsReal const &getMu() const { return *mu; }
 
 protected:
 
@@ -39,13 +43,13 @@ protected:
   RooRealProxy beta ;
   RooRealProxy mu ;
 
-  double evaluate() const ;
-  void computeBatch(cudaStream_t*, double* output, size_t nEvents, RooFit::Detail::DataMap const&) const;
-  inline bool canComputeBatchWithCuda() const { return true; }
+  double evaluate() const override ;
+  void doEval(RooFit::EvalContext &) const override;
+  inline bool canComputeBatchWithCuda() const override { return true; }
 
 private:
 
-  ClassDef(RooGamma,1) // Gaussian PDF
+  ClassDefOverride(RooGamma,1) // Gaussian PDF
 };
 
 #endif
