@@ -19,7 +19,6 @@ import sys
 from time import sleep
 from itertools import zip_longest
 
-
 def fileno(file_or_fd):
     """
     Look for 'fileno' attribute.
@@ -196,8 +195,13 @@ def isDirectoryKey(key):
     """
     Return True if the object, corresponding to the key, inherits from TDirectory
     """
+    import cppyy
+
     classname = key.GetClassName()
     cl = ROOT.gROOT.GetClass(classname)
+    if cl == cppyy.nullptr:
+        logging.warning("Unknown class to ROOT: " + classname)
+        return False
     return cl.InheritsFrom(ROOT.TDirectory.Class())
 
 
@@ -205,8 +209,13 @@ def isTreeKey(key):
     """
     Return True if the object, corresponding to the key, inherits from TTree
     """
+    import cppyy
+
     classname = key.GetClassName()
     cl = ROOT.gROOT.GetClass(classname)
+    if cl == cppyy.nullptr:
+        logging.warning("Unknown class to ROOT: " + classname)
+        return False
     return cl.InheritsFrom(ROOT.TTree.Class())
 
 
@@ -214,8 +223,13 @@ def isTHnSparseKey(key):
     """
     Return True if the object, corresponding to the key, inherits from THnSparse
     """
+    import cppyy
+
     classname = key.GetClassName()
     cl = ROOT.gROOT.GetClass(classname)
+    if cl == cppyy.nullptr:
+        logging.warning("Unknown class to ROOT: " + classname)
+        return False
     return cl.InheritsFrom(ROOT.THnSparse.Class())
 
 
@@ -789,11 +803,12 @@ REPLACE_HELP = "replace object if already existing"
 
 def _openBrowser(rootFile=None):
     browser = ROOT.TBrowser()
-    if ROOT.gSystem.InheritsFrom("TMacOSXSystem"):
+    if ROOT.gSystem.InheritsFrom("TMacOSXSystem") or browser.IsWeb():
         print("Press ctrl+c to exit.")
         try:
             while True:
-                ROOT.gSystem.ProcessEvents()
+                if ROOT.gROOT.IsInterrupted() or ROOT.gSystem.ProcessEvents():
+                    break
                 sleep(0.01)
         except (KeyboardInterrupt, SystemExit):
             pass
