@@ -1,45 +1,39 @@
 sap.ui.define([
    'rootui5/eve7/lib/GlViewer',
    'rootui5/eve7/lib/EveElements',
-   'rootui5/eve7/lib/OrbitControlsEve',
-   'rootui5/eve7/lib/OutlinePass',
+   'rootui5/eve7/lib/OutlinePassEve',
    'rootui5/eve7/lib/FXAAShader'
 ], function(GlViewer, EveElements) {
 
    "use strict";
 
-   function GlViewerThree(viewer_class) {
-      GlViewer.call(this, viewer_class);
-   }
+   let g_global_init_done = false;
 
-   GlViewerThree.prototype = Object.assign(Object.create(GlViewer.prototype), {
+   class GlViewerThree extends GlViewer {
 
-      constructor: GlViewerThree,
-
-      g_highlight_update: function(mgr) {
-         let sa = THREE.OutlinePass.selection_atts;
+      g_highlight_update(mgr) {
+         let sa = THREE.OutlinePassEve.selection_atts;
          let gs = mgr.GetElement(mgr.global_selection_id);
          let gh = mgr.GetElement(mgr.global_highlight_id);
 
          if (gs && gh) {
-            sa[0].visibleEdgeColor.setStyle(JSROOT.Painter.getColor(gs.fVisibleEdgeColor));
-            sa[0].hiddenEdgeColor.setStyle(JSROOT.Painter.getColor(gs.fHiddenEdgeColor));
-            sa[1].visibleEdgeColor.setStyle(JSROOT.Painter.getColor(gh.fVisibleEdgeColor));
-            sa[1].hiddenEdgeColor.setStyle(JSROOT.Painter.getColor(gh.fHiddenEdgeColor));
+            sa[0].visibleEdgeColor.setStyle(EVE.JSR.getColor(gs.fVisibleEdgeColor));
+            sa[0].hiddenEdgeColor.setStyle(EVE.JSR.getColor(gs.fHiddenEdgeColor));
+            sa[1].visibleEdgeColor.setStyle(EVE.JSR.getColor(gh.fVisibleEdgeColor));
+            sa[1].hiddenEdgeColor.setStyle(EVE.JSR.getColor(gh.fHiddenEdgeColor));
          }
-      },
+      }
 
-      init: function(controller) {
-         GlViewer.prototype.init.call(this, controller);
-         //super.init(controller);
+      init(controller) {
+         super.init(controller);
 
          this.creator = new EveElements(controller);
          // MT-RCORE indices now work, we probably don't need this option anymore.
-         // this.creator.useIndexAsIs = JSROOT.decodeUrl().has('useindx');
+         // this.creator.useIndexAsIs = EVE.JSR.decodeUrl().has('useindx');
          this.creator.useIndexAsIs = true;
 
-         if (!GlViewerThree.g_global_init_done) {
-            GlViewerThree.g_global_init_done = true;
+         if (!g_global_init_done) {
+            g_global_init_done = true;
 
             this.controller.mgr.RegisterSelectionChangeFoo(this.g_highlight_update.bind(this));
             this.g_highlight_update(this.controller.mgr);
@@ -53,42 +47,43 @@ sap.ui.define([
          this.setupThreejsDomAndEventHandlers();
 
          this.controller.glViewerInitDone();
-      },
+      }
 
-      cleanup: function() {
+      cleanup() {
          if (this.controller) this.controller.removeScenes();
          this.destroyThreejsRenderer();
-         GlViewer.prototype.cleanup.call(this);
-      },
+         super.cleanup();
+      }
 
       //==============================================================================
 
-      make_object: function(name) {
+      make_object(/*name*/) {
          return new THREE.Object3D;
-      },
+      }
 
-      get_top_scene: function() {
+      get_top_scene() {
          return this.scene;
-      },
+      }
+
+      get outline_map() { return this.outline_pass.id2obj_map; }
 
       //==============================================================================
       // THREE renderer creation, DOM/event handler setup, reset
       //==============================================================================
 
-      createThreejsRenderer: function() {
-         var w = this.get_width(), h = this.get_height();
+      createThreejsRenderer() {
+         let w = this.get_width(), h = this.get_height();
 
          // console.log("createThreejsRenderer", this.controller.kind, "w=", w, "h=", h);
 
          this.scene = new THREE.Scene();
          // this.scene.fog = new THREE.FogExp2( 0xaaaaaa, 0.05 );
 
-         if (this.controller.kind === "3D") {
+         if (this.controller.isEveCameraPerspective())
             this.camera = new THREE.PerspectiveCamera(75, w / h, 1, 5000);
-         }
-         else {
+         else
             this.camera = new THREE.OrthographicCamera(-w / 2, w / 2, -h / 2, h / 2, 0, 2000);
-         }
+
          this.scene.add(this.camera);
 
          this.rot_center = new THREE.Vector3(0, 0, 0);
@@ -107,30 +102,33 @@ sap.ui.define([
 
          // Lights are positioned in resetRenderer
 
-         this.point_lights = new THREE.Object3D;
-         this.point_lights.add(new THREE.PointLight(0xffffff, 0.7)); // R
-         this.point_lights.add(new THREE.PointLight(0xffffff, 0.7)); // G
-         this.point_lights.add(new THREE.PointLight(0xffffff, 0.7)); // B
+         this.point_lights = new THREE.Object3D();
+         this.point_lights.add(new THREE.DirectionalLight(0xffffff, 1)); // R
+         this.point_lights.add(new THREE.DirectionalLight(0xffffff, 1)); // G
+         this.point_lights.add(new THREE.DirectionalLight(0xffffff, 1)); // B
          this.scene.add(this.point_lights);
 
-         // var plane = new THREE.GridHelper(20, 20, 0x80d080, 0x8080d0);
+
+         // let plane = new THREE.GridHelper(20, 20, 0x80d080, 0x8080d0);
          // this.scene.add(plane);
 
          this.composer = new THREE.EffectComposer(this.renderer);
          this.composer.addPass(new THREE.RenderPass(this.scene, this.camera));
 
-         this.outline_pass = new THREE.OutlinePass(new THREE.Vector2(w, h), this.scene, this.camera);
+         this.outline_pass = new THREE.OutlinePassEve(new THREE.Vector2(w, h), this.scene, this.camera);
          this.outline_pass.edgeStrength = 5.5;
          this.outline_pass.edgeGlow = 0.7;
          this.outline_pass.edgeThickness = 1.5;
          this.outline_pass.usePatternTexture = false;
          this.outline_pass.downSampleRatio = 1;
          this.outline_pass.glowDownSampleRatio = 3;
+         this.outline_pass.id2obj_map = {};
+
 
          // This does not work ... seems it is not standard pass?
          // this.outline_pass.renderToScreen = true;
          // Tried hacking with this, but would apparently need to load it somehow, sigh.
-         // var copyPass = new ShaderPass( CopyShader );
+         // let copyPass = new ShaderPass( CopyShader );
          // this.composer.addPass( new THREE.ShaderPass(CopyShader) );
 
          this.composer.addPass(this.outline_pass);
@@ -140,19 +138,18 @@ sap.ui.define([
          this.fxaa_pass.renderToScreen = true;
 
          this.composer.addPass(this.fxaa_pass);
-      },
+      }
 
-      destroyThreejsRenderer: function() {
+      destroyThreejsRenderer() {
          if (this.renderer) {
             this.get_view().getDomRef().removeChild(this.renderer.domElement);
-            this.renderer.domElement.removeEventListener('mousemove', this.mousemove_func);
-            this.renderer.domElement.removeEventListener('mouseleave', this.mouseleave_func);
-            this.renderer.domElement.removeEventListener('mousedown', this.mousedown_func);
+            this.renderer.domElement.removeEventListener('pointermove', this.mousemove_func);
+            this.renderer.domElement.removeEventListener('pointerleave', this.mouseleave_func);
+            this.renderer.domElement.removeEventListener('pointerdown', this.mousedown_func);
             this.renderer.domElement.removeEventListener('dblclick', this.dblclick_func);
             window.removeEventListener('keydown', this.keydown_func);
          }
 
-         this.removeMouseupListener();
          this.removeMouseMoveTimeout();
          delete this.renderer;
          delete this.scene;
@@ -161,57 +158,67 @@ sap.ui.define([
             this.controls.dispose();
             delete this.controls;
          }
-      },
+      }
 
-      mouseMoveHandler: function(event) {
+      mouseMoveHandler(event) {
          if (event.movementX == 0 && event.movementY == 0)
             return;
 
-         this.removeMouseupListener();
-
          if (event.buttons === 0) {
             this.removeMouseMoveTimeout();
             this.mousemove_timeout = setTimeout(this.onMouseMoveTimeout.bind(this, event.offsetX, event.offsetY), this.controller.htimeout);
          } else {
             this.clearHighlight();
          }
-      },
+      }
 
-      mouseLeaveHandler: function(/* event */) {
+      mouseLeaveHandler(/* event */) {
          this.removeMouseMoveTimeout();
          this.clearHighlight();
-         this.removeMouseupListener();
-      },
+      }
 
-      mouseUpHandler: function(e0_buttons, event) {
-         this.removeMouseupListener();
-
-         if (e0_buttons == 1) {// Selection on mouseup without move
-            this.handleMouseSelect(event);
-         } else if (e0_buttons == 2) { // Context menu on delay without move
-            // Was needed for "on press with timeout"
-            // this.controls.resetMouseDown(event);
-
-         }
-      },
-
-      mouseDownHandler: function(event) {
+      mouseDownHandler(event) {
          this.removeMouseMoveTimeout();
-         if (event.buttons != 1 && event.buttons != 2) this.clearHighlight();
-         this.removeMouseupListener();
-
-         if ((event.buttons == 1) && this.renderer) {
-            this.mouseup_listener = this.mouseUpHandler.bind(this, event.buttons);
-            this.renderer.domElement.addEventListener('mouseup', this.mouseup_listener);
+         if (event.buttons != 1 && event.buttons != 2)
+            this.clearHighlight();
+         else if (this.renderer) {
+            // keep track which buttons and where are clicked
+            this.click_event = event;
+            this.click_buttons = event.buttons;
+            this.click_intersect = this.getIntersectAt(event.offsetX, event.offsetY);
          }
-      },
+      }
 
-      dblClickHandler: function(/* event */) {
+      clearClickedButtons() {
+         delete this.click_buttons;
+         delete this.click_intersect;
+      }
+
+      processControlEnd() {
+         if (this.click_buttons == 1) {
+            // handle left mouse button click
+            if (this.click_intersect) {
+               let c = this.getCtrlRec(this.click_intersect.object);//this.click_intersect.object.get_ctrl();
+               c.event = this.click_event;
+               c.elementSelected(c.extractIndex(this.click_intersect));
+               this.highlighted_scene = this.click_intersect.object.scene;
+            } else {
+               // XXXX HACK - handlersMIR senders should really be in the mgr
+
+               this.controller.created_scenes[0].processElementSelected(null, [], this.click_event);
+            }
+         } else if (this.click_buttons == 2) {
+            let intersect = this.click_intersect;
+            EVE.JSR.createMenu(this.click_event, this).then(menu => this.showContextMenu(intersect, menu));
+         }
+      }
+
+      dblClickHandler(/* event */) {
          if (this.controller.dblclick_action == "Reset")
             this.resetThreejsRenderer();
-      },
+      }
 
-      keyDownHandler: function(event) {
+      keyDownHandler(event) {
          let handled = true;
 
          if (event.key == "t") {
@@ -253,10 +260,9 @@ sap.ui.define([
 
             this.render();
          }
-      },
+      }
 
-
-      setupThreejsDomAndEventHandlers: function() {
+      setupThreejsDomAndEventHandlers() {
          this.get_view().getDomRef().appendChild(this.renderer.domElement);
 
          // Setup tooltip
@@ -266,11 +272,6 @@ sap.ui.define([
          this.ttip.appendChild(this.ttip_text);
          this.get_view().getDomRef().appendChild(this.ttip);
 
-         // Setup controls
-         this.controls = new THREE.OrbitControlsEve(this.camera, this.get_view().getDomRef());
-
-         this.controls.addEventListener('change', this.render.bind(this));
-
          // Setup some event pre-handlers
          this.mousemove_func = this.mouseMoveHandler.bind(this);
          this.mouseleave_func = this.mouseLeaveHandler.bind(this);
@@ -278,20 +279,31 @@ sap.ui.define([
          this.dblclick_func = this.dblClickHandler.bind(this);
          this.keydown_func = this.keyDownHandler.bind(this);
 
-         this.renderer.domElement.addEventListener('mousemove', this.mousemove_func);
-         this.renderer.domElement.addEventListener('mouseleave', this.mouseleave_func);
-         this.renderer.domElement.addEventListener('mousedown', this.mousedown_func);
+         this.renderer.domElement.addEventListener('pointermove', this.mousemove_func);
+         this.renderer.domElement.addEventListener('pointerleave', this.mouseleave_func);
+         this.renderer.domElement.addEventListener('pointerdown', this.mousedown_func);
          this.renderer.domElement.addEventListener('dblclick', this.dblclick_func);
 
          // Key-handlers go on window ...
          window.addEventListener('keydown', this.keydown_func);
 
+         // Setup controls
+         this.controls = new THREE.OrbitControls(this.camera, this.get_view().getDomRef());
+         this.controls.addEventListener('change', () => {
+            this.clearClickedButtons();
+            this.render();
+         })
+         this.controls.addEventListener('end', () => {
+            this.processControlEnd();
+            this.clearClickedButtons();
+         });
+
          // This will also call render().
          this.resetThreejsRenderer();
-      },
+      }
 
       /** Reset camera, lights based on scene bounding box. */
-      resetThreejsRenderer: function() {
+      resetThreejsRenderer() {
          let sbbox = new THREE.Box3();
          sbbox.setFromObject(this.scene);
 
@@ -341,7 +353,8 @@ sap.ui.define([
             this.camera.top = ey;
             this.camera.bottom = -ey;
 
-            this.controls.resetOrthoPanZoom();
+            if (typeof this.controls.resetOrthoPanZoom == 'function')
+               this.controls.resetOrthoPanZoom();
 
             this.controls.screenSpacePanning = true;
             this.controls.enableRotate = false;
@@ -353,22 +366,27 @@ sap.ui.define([
          this.composer.reset();
 
          this.controls.update();
-      },
+      }
 
 
       //==============================================================================
 
-      render: function() {
+      render() {
+         // AMT check if controller is attached in the splitter
+         let v = this.get_manager().GetElement(this.controller.eveViewerId);
+         if (!v.fRnrSelf)
+            return;
+
          // Render through composer:
          this.composer.render(this.scene, this.camera);
 
          // or directly through renderer:
          // this.renderer.render( this.scene, this.camera );
-      },
+      }
 
       //==============================================================================
 
-      onResizeTimeout: function() {
+      onResizeTimeout() {
          let w = this.get_width();
          let h = this.get_height();
 
@@ -389,7 +407,7 @@ sap.ui.define([
          this.composer.reset();
          this.controls.update();
          this.render();
-      },
+      }
 
 
       //==============================================================================
@@ -400,24 +418,34 @@ sap.ui.define([
       // Highlight & Mouse move timeout handling
       //------------------------------------------------------------------------------
 
-      clearHighlight: function() {
+      clearHighlight() {
          if (this.highlighted_scene) {
             this.highlighted_scene.clearHighlight(); // XXXX should go through manager
             this.highlighted_scene = 0;
 
             this.ttip.style.display = "none";
          }
-      },
+      }
 
-      removeMouseMoveTimeout: function() {
+      removeMouseMoveTimeout() {
          if (this.mousemove_timeout) {
             clearTimeout(this.mousemove_timeout);
             delete this.mousemove_timeout;
          }
-      },
+      }
+
+      getCtrlRec(io)
+      {
+         if (io.get_ctrl)
+         return io.get_ctrl();
+         else if (io.parent)
+         return this.getCtrlRec(io.parent)
+         else
+         return null;
+      }
 
       /** Get three.js intersect object at specified mouse position */
-      getIntersectAt: function(x, y) {
+      getIntersectAt(x, y) {
          let w = this.get_width();
          let h = this.get_height();
 
@@ -428,28 +456,37 @@ sap.ui.define([
          this.raycaster.setFromCamera(mouse, this.camera);
 
          let intersects = this.raycaster.intersectObjects(this.scene.children, true);
-
-         for (let i = 0; i < intersects.length; ++i) {
-            if (intersects[i].object.get_ctrl && intersects[i].object.visible) {
-               intersects[i].mouse = mouse;
-               intersects[i].w = w;
-               intersects[i].h = h;
-               return intersects[i];
+         for (let i = 0; i < intersects.length; ++i)
+         {
+            let ctrl = this.getCtrlRec(intersects[i].object);
+            if (intersects[i].object.visible && ctrl)
+            {
+               if (ctrl && ctrl.obj3d && ctrl.obj3d.eve_el)
+               {
+                  let el = ctrl.obj3d.eve_el;
+                  if (el && el.fPickable)
+                  {
+                     intersects[i].mouse = mouse;
+                     intersects[i].w = w;
+                     intersects[i].h = h;
+                     return intersects[i];
+                  }
+               }
             }
          }
-      },
+         return null;
+      }
 
-      onMouseMoveTimeout: function(x, y) {
+      onMouseMoveTimeout(x, y) {
          delete this.mousemove_timeout;
 
-         var intersect = this.getIntersectAt(x, y);
-
+         let intersect = this.getIntersectAt(x, y);
          if (!intersect)
             return this.clearHighlight();
 
-         var c = intersect.object.get_ctrl();
+         let c = this.getCtrlRec(intersect.object);//intersect.object.get_ctrl();
 
-         var mouse = intersect.mouse;
+         let mouse = intersect.mouse;
 
          c.elementHighlighted(c.extractIndex(intersect));
 
@@ -479,12 +516,16 @@ sap.ui.define([
          }
 
          this.ttip.style.display = "block";
-      },
-      remoteToolTip: function(msg) {
-         this.ttip_text.innerHTML = msg;
-         if (this.highlighted_scene) this.ttip.style.display = "block";
-      },
-      getRelativeOffsets: function(elem) {
+      }
+
+      remoteToolTip(msg) {
+         if (this.ttip_text && this.ttip_text.innerHTML)
+            this.ttip_text.innerHTML = msg;
+         if (this.highlighted_scene && this.ttip)
+            this.ttip.style.display = "block";
+      }
+
+      getRelativeOffsets(elem) {
          // Based on:
          // https://stackoverflow.com/questions/3000887/need-to-calculate-offsetright-in-javascript
 
@@ -503,33 +544,22 @@ sap.ui.define([
          }
 
          return r;
-      },
+      }
 
       //------------------------------------------------------------------------------
       // Mouse button handlers, selection, context menu
       //------------------------------------------------------------------------------
 
-      removeMouseupListener: function() {
-         if (this.mouseup_listener) {
-            if (this.render)
-               this.renderer.domElement.removeEventListener('mouseup', this.mouseup_listener);
-            delete this.mouseup_listener;
-         }
-      },
-
-      showContextMenu: function(event, menu) {
+      showContextMenu(intersect, menu) {
          // console.log("GLC::showContextMenu", this, menu)
 
-         // See js/scripts/JSRootPainter.jquery.js JSROOT.Painter.createMenu(), menu.add()
-
-
-         var intersect = this.getIntersectAt(event.offsetX, event.offsetY);
+         // See js/modules/gui/menu.mjs createMenu(), menu.add()
 
          menu.add("header:Context Menu");
 
          if (intersect) {
-            if (intersect.object.eve_el)
-               menu.add("Browse to " + (intersect.object.eve_el.fName || "element"), intersect.object.eve_el.fElementId, this.controller.invokeBrowseOf.bind(this.controller));
+            let el = intersect.object.get_ctrl().obj3d.eve_el;
+            menu.add("Browse to " + (el.fName || "element"), el.fElementId, this.controller.invokeBrowseOf.bind(this.controller));
          }
 
          menu.add("Reset camera", this.resetThreejsRenderer);
@@ -543,18 +573,17 @@ sap.ui.define([
          menu.add("Baz", 'baz', fff);
          menu.add("endsub:");
 
-         menu.show(event);
-      },
+         menu.show();
+      }
 
-      defaultContextMenuAction: function(arg) {
+      defaultContextMenuAction(arg) {
          console.log("GLC::defaultContextMenuAction", this, arg);
-      },
-
-      handleMouseSelect: function(event) {
-         var intersect = this.getIntersectAt(event.offsetX, event.offsetY);
+      }
 
+      handleMouseSelect(event) {
+         let intersect = this.getIntersectAt(event.offsetX, event.offsetY);
          if (intersect) {
-            var c = intersect.object.get_ctrl();
+            let c = intersect.object.get_ctrl();
             c.event = event;
             c.elementSelected(c.extractIndex(intersect));
             this.highlighted_scene = intersect.object.scene;
@@ -563,9 +592,9 @@ sap.ui.define([
 
             this.controller.created_scenes[0].processElementSelected(null, [], event);
          }
-      },
+      }
 
-   });
+   } // class GlViewerThree
 
    //==============================================================================
    // THREE.js hacks
@@ -578,27 +607,27 @@ sap.ui.define([
 
          if (!this.attributes.position || !this.index) return;
 
-         var index = this.index;
-         var attributes = this.attributes;
-         var positions = attributes.position.array;
+         let index = this.index;
+         let attributes = this.attributes;
+         let positions = attributes.position.array;
          if (attributes.normal === undefined) {
             this.setAttribute('normal', new THREE.BufferAttribute(new Float32Array(positions.length), 3));
          } else {
             // reset existing normals to zero
-            var array = attributes.normal.array;
-            for (var i = 0, il = array.length; i < il; i++) {
+            let array = attributes.normal.array;
+            for (let i = 0, il = array.length; i < il; i++) {
                array[i] = 0;
             }
          }
-         var normals = attributes.normal.array;
+         let normals = attributes.normal.array;
 
-         var vA, vB, vC;
-         var pA = new THREE.Vector3(), pB = new THREE.Vector3(), pC = new THREE.Vector3();
-         var cb = new THREE.Vector3(), ab = new THREE.Vector3();
+         let vA, vB, vC;
+         let pA = new THREE.Vector3(), pB = new THREE.Vector3(), pC = new THREE.Vector3();
+         let cb = new THREE.Vector3(), ab = new THREE.Vector3();
 
-         var indices = index.array;
+         let indices = index.array;
 
-         for (var i = start, i_end = start + count; i < i_end; i += 3) {
+         for (let i = start, i_end = start + count; i < i_end; i += 3) {
 
             vA = indices[i + 0] * 3;
             vB = indices[i + 1] * 3;
@@ -631,6 +660,7 @@ sap.ui.define([
 
       };
 
+
    }
 
    //==============================================================================
